@@ -225,10 +225,10 @@ class sauserprefs extends rcube_plugin
 
         // output table sorting prefs
         $sorts = $this->rcube->config->get('sauserprefs_sort', array());
-        if (!array_key_exists('#spam-langs-table', $sorts)) {
+        if (!isset($sorts['#spam-langs-table'])) {
             $sorts['#spam-langs-table'] = array(0, 'true');
         }
-        if (!array_key_exists('#address-rules-table', $sorts)) {
+        if (!isset($sorts['#address-rules-table'])) {
             $sorts['#address-rules-table'] = array(1, 'true');
         }
         $this->rcube->output->set_env('sauserprefs_sort', $sorts);
@@ -482,9 +482,7 @@ class sauserprefs extends rcube_plugin
 
             $new_prefs = array();
 
-            $emails = $this->_gen_email_arr($args['record']);
-            $emails = array_unique($emails);
-            foreach ($emails as $email) {
+            foreach ($this->_gen_email_arr($args['record']) as $email) {
                 if (!in_array($email, $existing_addresses)) {
                     $new_prefs['addresses'][] = array('field' => self::map_pref_name('whitelist_from'), 'value' => $email, 'action' => 'INSERT');
                 }
@@ -527,7 +525,6 @@ class sauserprefs extends rcube_plugin
             $contacts = $this->rcube->get_address_book($args['source']);
             foreach ($args['id'] as $id) {
                 $emails = $this->_gen_email_arr($contacts->get_record($id, true));
-                $emails = array_unique($emails);
                 foreach ($emails as $email) {
                     if (in_array($email, $existing_addresses)) {
                         $new_prefs['addresses'][] = array('field' => self::map_pref_name('whitelist_from'), 'value' => $email, 'action' => 'DELETE');
@@ -605,7 +602,7 @@ class sauserprefs extends rcube_plugin
         $prefs = $this->storage->load_prefs($user);
 
         // sort address rules
-        if (is_array($prefs['addresses'])) {
+        if (isset($prefs['addresses']) && is_array($prefs['addresses'])) {
             usort($prefs['addresses'], array($this, 'sort_addresses'));
         }
 
@@ -707,7 +704,7 @@ class sauserprefs extends rcube_plugin
                         $button = '';
                         $checkbox_display = array();
 
-                        if ($attrib['lang_list_buttons'] == '1') {
+                        if (isset($attrib['lang_list_buttons']) && $attrib['lang_list_buttons'] == '1') {
                             $button_type = in_array($lang_code, $locales_langs) ? 'enabled' : 'disabled';
                             $button = $this->rcube->output->button(array('command' => 'plugin.sauserprefs.message_lang', 'prop' => $lang_code, 'type' => 'link', 'class' => 'lang-' . $button_type, 'id' => 'spam_lang_' . $i, 'title' => 'sauserprefs.' . $button_type, 'content' => ' '));
                             $checkbox_display = array('style' => 'display: none;');
@@ -1048,7 +1045,7 @@ class sauserprefs extends rcube_plugin
                 $field_id = 'rcmbtn_add_address';
                 $button_addaddress = $this->rcube->output->button(array('id' => $field_id, 'command' => 'plugin.sauserprefs.addressrule_add', 'type' => 'input', 'class' => 'button', 'label' => 'sauserprefs.addrule'));
 
-                $blocks['main']['intro'] .= html::div('address-input grouped', $input_spamaddressrule->show() . $input_spamaddress->show() . $button_addaddress);
+                $blocks['main']['intro'] .= html::div('address-input grouped form-inline', $input_spamaddressrule->show() . $input_spamaddress->show() . $button_addaddress);
 
                 $import = !empty($this->addressbook_import) ? $this->rcube->output->button(array('class' => 'import', 'href' => '#', 'onclick' => 'return ' . rcmail_output::JS_OBJECT_NAME . '.sauserprefs_address_import_dialog();', 'type' => 'link', 'label' => 'sauserprefs.importaddresses', 'title' => 'sauserprefs.importfromaddressbook')) : '';
                 $delete_all = $this->rcube->output->button(array('class' => 'delete-all', 'command' => 'plugin.sauserprefs.whitelist_delete_all', 'type' => 'link', 'label' => 'sauserprefs.deleteall', 'title' => 'sauserprefs.deletealltip'));
@@ -1175,7 +1172,7 @@ class sauserprefs extends rcube_plugin
                         $table->add('title', $row['title']);
                     }
 
-                    $table->add($row['content_attribs'], $row['content']);
+                    $table->add(isset($row['content_attribs']) ? $row['content_attribs'] : null, $row['content']);
 
                     if (isset($row['help'])) {
                         $table->add('help', $row['help']);
@@ -1186,7 +1183,7 @@ class sauserprefs extends rcube_plugin
             }
 
             if (!empty($content)) {
-                $out .= html::tag('fieldset', $class, html::tag('legend', null, $block['name']) . $block['intro'] . $content);
+                $out .= html::tag('fieldset', $class, html::tag('legend', null, $block['name']) . (isset($block['intro']) ? $block['intro'] : '') . $content);
             }
         }
 
@@ -1209,6 +1206,10 @@ class sauserprefs extends rcube_plugin
             case "whitelist_to":
                 $fieldtxt = rcube_utils::rep_specialchars_output($this->gettext('whitelist_to'));
                 break;
+            default:
+                // for default/placeholder row
+                $fieldtxt = '';
+                $class = '';
         }
 
         $row_attrib = !isset($field) ? array_merge($row_attrib, array('style' => 'display: none;')) : array_merge($row_attrib, array('class' => $field));
@@ -1236,7 +1237,7 @@ class sauserprefs extends rcube_plugin
         for ($i = $config['min']; $i <= $config['max']; $i += $config['increment']) {
             $vals[number_format($i, 5, '.', '')] = array('val' => number_format($i, $decPlaces, '.', ''), 'text' => number_format($i, $decPlaces, $locale_info['decimal_point'], ''));
         }
-        if (array_key_exists('extra', $config)) {
+        if (isset($config['extra'])) {
             foreach ($config['extra'] as $extra) {
                 $decPlaces = self::decimal_places($extra['increment'], $locale_info['decimal_point']);
                 for ($i = $extra['min']; $i <= $extra['max']; $i += $extra['increment']) {
@@ -1311,16 +1312,14 @@ class sauserprefs extends rcube_plugin
         foreach ($contact as $key => $value) {
             if (preg_match('/^email(:(.+))?$/i', $key, $matches)) {
                 foreach ((array) $value as $subkey => $subval) {
-                    if ($matches[2]) {
-                        $emails[$matches[2] . $subkey] = $subval;
-                    }
-                    else {
-                        $emails['email' . $subkey] = $subval;
+                    if (!empty($subval)) {
+                        $emails[] = $subval;
                     }
                 }
             }
         }
 
+        $emails = array_unique($emails);
         return $emails;
     }
 
